use std::collections::HashMap;

use serde::{Serialize, Deserialize};

pub struct GridCell {
    pub north_wall: String,
    pub south_wall: String,
    pub east_wall:  String,
    pub west_wall:  String,
}

pub enum Direction {
    North,
    South,
    East,
    West,
}

#[derive(Serialize, Deserialize, Debug)]
struct MapFile {
    default: String,
    overlays: HashMap<String, String>,
}

static example_map_json5: &str = r##"
{
default: "_________________________________\
| _,_____ |_+ +_|,|,+ ____|_____|\
| | ____+   + +     | |,__+_____|\
| | |       +_+_____| _,_ |_____|\
| |,|   _,_     ______| | ______|\
|__,___ + +____ +_______| |_____|\
|__ __+ | +,+_| ____+_+   ______|\
|_| |__ |,|__,| +_____|   |_____|\
|__ __+ | +   | _____,___ ___ __|\
|_| |__ | |__,| | | ____| |,| +_|\
|,_ _,+ | +   | +_|_| _,_,_ ___,|\
| |,|   |_|___| |_+_+ |_|_| |___|\
|,_ _,_ _,_ ___ +_|,| +_| ___ __|\
| |_| | | | | + _,___,+_| | | | |\
|_____| |_| |_| |__,|_|   | | | |\
|_____+___________________|_|_+_|",
overlays: {
  "indoors": "_________________________________\
| _______ ########### __________|\
| #######   ######### #####_____|\
| ###       ######### ___ |_____|\
| ###   ___     ______### ______|\
|______ ###____ ######### |_____|\
|###### ####### ____###   ______|\
|_###__ ####### #######   |_____|\
|###### ####### _________ ___ __|\
|_###__ ####### ######### ### ##|\
|###### ####### ##### _____ ____|\
| ###   ####### ##### ##### ####|\
|__ ___ ___ ___ ##### ### ___ __|\
|##_### ### ### ______### ### ##|\
|###### ### ### ######    ### ##|\
|######___________________###_##|",
},

}"##;


pub fn example_map() -> Vec<Vec<GridCell>> {
    let unused_map: MapFile = serde_json5::from_str(example_map_json5).unwrap();

    return vec![
        vec![
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "door".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "door".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "door".to_string(),
                west_wall:  "door".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "door".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "door".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "door".to_string(),
                east_wall:  "door".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "door".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "no".to_string(),
            },
        ], vec![
            GridCell{
                north_wall: "no".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "door".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "door".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "no".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "door".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "door".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "no".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "door".to_string(),
                west_wall:  "door".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "door".to_string(),
            },
            GridCell{
                north_wall: "door".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "door".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "no".to_string(),
                south_wall: "no".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "door".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "yes".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "door".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "door".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "no".to_string(),
                west_wall:  "no".to_string(),
            },
            GridCell{
                north_wall: "yes".to_string(),
                south_wall: "yes".to_string(),
                east_wall:  "yes".to_string(),
                west_wall:  "no".to_string(),
            },
        ],
    ];
    
}
